@@ -1,15 +1,13 @@
 ###############################################################################
 #
-# This file copyright (c) 2001 by Randy J. Ray, all rights reserved
+# This file copyright (c) 2001-2009 Randy J. Ray, all rights reserved
 #
 # Copying and distribution are permitted under the terms of the Artistic
-# License as distributed with Perl versions 5.005 and later. See
-# http://www.opensource.org/licenses/artistic-license.php
+# License 2.0 (http://www.opensource.org/licenses/artistic-license-2.0.php) or
+# the GNU LGPL (http://www.opensource.org/licenses/lgpl-2.1.php).
 #
 ###############################################################################
 #
-#   $Id: Procedure.pm,v 1.12 2004/12/14 10:01:24 rjray Exp $
-#
 #   Description:    This class abstracts out all the procedure-related
 #                   operations from the RPC::XML::Server class
 #
@@ -40,7 +38,7 @@
 
 package RPC::XML::Procedure;
 
-use 5.005;
+use 5.006001;
 use strict;
 use vars qw($VERSION);
 use subs qw(new is_valid name code signature help version hidden
@@ -50,7 +48,9 @@ use subs qw(new is_valid name code signature help version hidden
 use AutoLoader 'AUTOLOAD';
 require File::Spec;
 
-$VERSION = do { my @r=(q$Revision: 1.12 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
+use Scalar::Util 'blessed';
+
+$VERSION = '1.17';
 
 ###############################################################################
 #
@@ -191,6 +191,7 @@ sub make_sig_table
 # These are basic accessor/setting functions for the various attributes
 #
 sub name      { $_[0]->{name}; } # "name" cannot be changed at this level
+sub namespace { $_[0]->{namespace} || ''; } # Nor can "namespace"
 sub help      { $_[1] and $_[0]->{help}    = $_[1]; $_[0]->{help};    }
 sub version   { $_[1] and $_[0]->{version} = $_[1]; $_[0]->{version}; }
 sub hidden    { $_[1] and $_[0]->{hidden}  = $_[1]; $_[0]->{hidden};  }
@@ -358,6 +359,14 @@ persist. However, the other elements may be used in the creation of a new
 object, which may then be added to the server, if the name absolutely must
 change.
 
+=item namespace
+
+If the procedure object was created from a file, or if the instantiation
+included namespace information, this accessor will return the namespace that
+the underlying code executes in. Otherwise, it returns an empty string. This
+cannot be altered (even if the B<code> method is used to replace the code
+routine).
+
 =item code([NEW])
 
 Returns or sets the code-reference that will receive calls as marshalled by
@@ -432,10 +441,10 @@ passed in PARAMLIST. The SERVER argument should be an object derived from the
 B<RPC::XML::Server> class. For some types of procedure objects, this becomes
 the first argument of the parameter list to simulate a method call as if it
 were on the server object itself. The return value should be a data object
-(possible a B<RPC::XML::fault>), but may not always be pre-encoded. This
-method is generally used in the C<dispatch> method of the server class, where
-the return value is subsequently wrapped within a B<RPC::XML::response>
-object.
+(possibly a B<RPC::XML::fault>), but may not always be pre-encoded. Errors
+trapped in C<$@> are converted to fault objects. This method is generally used
+in the C<dispatch> method of the server class, where the return value is
+subsequently wrapped within a B<RPC::XML::response> object.
 
 =item reload
 
@@ -459,6 +468,14 @@ protected, and the same care should be taken before altering any of them:
 When the method was loaded from a file, this key contains the path to the file
 used.
 
+=item namespace
+
+If the code is loaded from a file, this hash key will reflect what namespace
+the code executes in. If the file specified a namespace, that is the value
+you will get (any occurrence of C<.> in the specified namespace will have been
+converted to C<::>). If no explicit namespace was provided, the namespace
+of the class you called B<new> from will be used. See L</"Namespaces">.
+
 =item mtime
 
 When the method was loaded from a file, this key contains the
@@ -495,11 +512,12 @@ other packages than this one, or useful in other contexts than this one.
 
 The lightweight DTD for the layout can be summarized as:
 
-        <!ELEMENT  proceduredef  (name, version?, hidden?, signature+,
-                                  help?, code)>
-        <!ELEMENT  methoddef  (name, version?, hidden?, signature+,
-                               help?, code)>
+        <!ELEMENT  proceduredef  (name, namespace?, version?, hidden?,
+                                  signature+, help?, code)>
+        <!ELEMENT  methoddef     (name, namespace?, version?, hidden?,
+                                  signature+, help?, code)>
         <!ELEMENT  name       (#PCDATA)>
+        <!ELEMENT  namespace  (#PCDATA)>
         <!ELEMENT  version    (#PCDATA)>
         <!ELEMENT  hidden     EMPTY>
         <!ELEMENT  signature  (#PCDATA)>
@@ -507,8 +525,8 @@ The lightweight DTD for the layout can be summarized as:
         <!ELEMENT  code       (#PCDATA)>
         <!ATTLIST  code       language (#PCDATA)>
 
-The containing tag is always one of C<E<lt>methoddefE<gt>> or
-C<E<lt>proceduredefE<gt>>. The tags that specify name, signatures and the code
+The containing tag is always one of C<< <methoddef> >> or
+C<< <proceduredef> >>. The tags that specify name, signatures and the code
 itself must always be present. Some optional information may also be
 supplied. The "help" text, or what an introspection API would expect to use to
 document the method, is also marked as optional.  Having some degree of
@@ -517,7 +535,7 @@ however.
 
 The default methods that this package provides are turned into XPL files by
 the B<make_method> tool (see L<make_method>). The final forms of these may
-serve as direct examples of what the file should look like.
+serve as examples of what the file should look like.
 
 =item Information used only for book-keeping
 
@@ -533,16 +551,16 @@ to do so.
 =item The information crucial to the method
 
 The name, signatures and code must be present for obvious reasons. The
-C<E<lt>nameE<gt>> tag tells the server what external name this procedure is
-known by. The C<E<lt>signatureE<gt>> tag, which may appear more than once,
+C<< <name> >> tag tells the server what external name this procedure is
+known by. The C<< <signature> >> tag, which may appear more than once,
 provides the definition of the interface to the function in terms of what
 types and quantity of arguments it will accept, and for a given set of
 arguments what the type of the returned value is. Lastly is the
-C<E<lt>codeE<gt>> tag, without which there is no procedure to remotely call.
+C<< <code> >> tag, without which there is no procedure to remotely call.
 
 =item Why the <code> tag allows multiple languages
 
-Note that the C<E<lt>codeE<gt>> tag is the only one with an attribute, in this
+Note that the C<< <code> >> tag is the only one with an attribute, in this
 case "language". This is designed to allow for one XPL file to provide a given
 method in multiple languages. Why, one might ask, would there be a need for
 this?
@@ -558,7 +576,9 @@ implementations in all four of the above languages, the name, help text,
 signature and even hidden status would likely be identical. So, why not share
 the non-language-specific elements in the spirit of re-use?
 
-=item The "make_method" utility
+=back
+
+=head2 The C<make_method> Utility
 
 The utility script C<make_method> is provided as a part of this software
 suite. It allows for the automatic creation of XPL files from either
@@ -569,7 +589,26 @@ automatic generation of XPL files and their delivery as a part of the normal
 Perl module build process. Using this tool is highly recommended over managing
 XPL files directly. For the full details, see L<make_method>.
 
-=back
+=head1 NAMESPACES
+
+As default behavior, Perl code that is passed to C<eval> when a XPL file is
+loaded gets put into the same namespace as the package used to load the XPL.
+It is not an issue when you create your own B<RPC::XML::Procedure> (or
+B<::Method> or B<::Function>) objects, as the code is already instantiated
+into a given namespace.  This can be important if your code expects to call
+routines in other loaded packages, utilize package-level globals, etc.
+
+To give developers control over the namespace in XPL code, a new optional
+tag C<< <namespace> >> was added in the 0.65 release. If this tag is present
+in the XPL being read, it defines the namespace that the C<< <code> >> block
+is evaluated in.
+
+The value of the namespace tag is a string providing the namespace in either
+the Perl-style of hierarchy parts separated by C<::>, or the style used by
+Java, Perl6, etc., in which the parts are separated by C<.>. The latter
+form is converted to Perl style for the evaluation of the code. If there is
+no namespace declaration in a XPL file, the namespace of the class that
+loads the XPL is used.
 
 =head1 DIAGNOSTICS
 
@@ -583,11 +622,53 @@ Moving the method management to a separate class adds a good deal of overhead
 to the general system. The trade-off in reduced complexity and added
 maintainability should offset this.
 
-=head1 LICENSE
+=head1 BUGS
 
-This module is licensed under the terms of the Artistic License that covers
-Perl. See <http://www.opensource.org/licenses/artistic-license.php> for the
-license.
+Please report any bugs or feature requests to
+C<bug-rpc-xml at rt.cpan.org>, or through the web interface at
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=RPC-XML>. I will be
+notified, and then you'll automatically be notified of progress on
+your bug as I make changes.
+
+=head1 SUPPORT
+
+=over 4
+
+=item * RT: CPAN's request tracker
+
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=RPC-XML>
+
+=item * AnnoCPAN: Annotated CPAN documentation
+
+L<http://annocpan.org/dist/RPC-XML>
+
+=item * CPAN Ratings
+
+L<http://cpanratings.perl.org/d/RPC-XML>
+
+=item * Search CPAN
+
+L<http://search.cpan.org/dist/RPC-XML>
+
+=item * Source code on GitHub
+
+L<http://github.com/rjray/rpc-xml/tree/master>
+
+=back
+
+=head1 COPYRIGHT & LICENSE
+
+This file and the code within are copyright (c) 2009 by Randy J. Ray.
+
+Copying and distribution are permitted under the terms of the Artistic
+License 2.0 (L<http://www.opensource.org/licenses/artistic-license-2.0.php>) or
+the GNU LGPL 2.1 (L<http://www.opensource.org/licenses/lgpl-2.1.php>).
+
+=head1 CREDITS
+
+The B<XML-RPC> standard is Copyright (c) 1998-2001, UserLand Software, Inc.
+See <http://www.xmlrpc.com> for more information about the B<XML-RPC>
+specification.
 
 =head1 SEE ALSO
 
@@ -595,7 +676,7 @@ L<RPC::XML::Server>, L<make_method>
 
 =head1 AUTHOR
 
-Randy J. Ray <rjray@blackperl.com>
+Randy J. Ray C<< <rjray@blackperl.com> >>
 
 =cut
 
@@ -829,10 +910,10 @@ sub load_XPL_file
     # So these don't end up undef, since they're optional elements
     $data->{hidden} = 0; $data->{version} = ''; $data->{help} = '';
     $data->{called} = 0;
-    open(F, "< $file");
-    return "$me: Error opening $file for reading: $!" if ($?);
+    open(F, "< $file") or return "$me: Error opening $file for reading: $!";
     $P = XML::Parser
-        ->new(Handlers => {Char  => sub { $accum .= $_[1] },
+        ->new(ErrorContext => 1,
+              Handlers => {Char  => sub { $accum .= $_[1] },
                            Start => sub { %attr = splice(@_, 2) },
                            End   =>
                            sub {
@@ -870,12 +951,29 @@ sub load_XPL_file
     return "$me: Error creating XML::Parser object" unless $P;
     # Trap any errors
     eval { $P->parse(*F) };
+    close(F);
     return "$me: Error parsing $file: $@" if $@;
 
     # Try to normalize $codetext before passing it to eval
-    my $class = __PACKAGE__; # token won't expand in the s/// below
+
+    # First step is set the namespace the code will live in. The default is
+    # the package that we're in (be it ::Procedure, ::Method, etc.). If they
+    # specify one, use it instead.
+    if ($data->{namespace})
+    {
+        # Fudge a little and let them '.' as a synonym for '::' in the
+        # namespace hierarchy.
+        $data->{namespace} =~ s{\.}{::}g;
+    }
+    else
+    {
+        $data->{namespace} = __PACKAGE__;
+    }
+
+    # Next step is to munge away any actual subroutine name so that the eval
+    # yields an anonymous sub. Also insert the namespace declaration.
     ($codetext = $data->{code}) =~
-        s/sub[\s\n]+([\w:]+)?[\s\n]*\{/sub \{ package $class; /;
+        s/sub[\s\n]+([\w:]+)?[\s\n]*\{/sub \{ package $data->{namespace}; /;
     $code = eval $codetext;
     return "$me: Error creating anonymous sub: $@" if $@;
 
@@ -942,15 +1040,19 @@ sub call
 
     # Now take a deep breath and call the method with the arguments
     eval { $response = $self->{code}->(@params); };
-    # Report a Perl-level error/failure if it occurs
-    return RPC::XML::fault->new(302, "Method $name returned error: $@") if $@;
+    # On failure, propagate user-generated RPC::XML::fault exceptions, or
+    # transform Perl-level error/failure into such an object
+    if ($@)
+    {
+        return (blessed $@ and $@->isa('RPC::XML::fault')) ?
+            $@ : RPC::XML::fault->new(302, "Method $name returned error: $@");
+    }
 
     $self->{called}++ unless $noinc;
     # Create a suitable return value
-    if ((! ref($response)) && UNIVERSAL::can("RPC::XML::$resptype", 'new'))
+    if ((! ref($response)) && "RPC::XML::$resptype"->can('new'))
     {
-        my $class = "RPC::XML::$resptype";
-        $response = $class->new($response);
+        $response = "RPC::XML::$resptype"->new($response);
     }
 
     $response;
